/*
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2020, 2021. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 */
/// <reference types="node" />
import * as fs from 'fs';
import { ZappDocumentInstance } from './zapp-document-instance';
/**
 * Class that finds, loads, and caches instances of zapp files in memory.
 * Manages file watchers that invalidate cache entries if a file was modified.
 */
export declare class ZappDocumentsCache {
    /** maps absolute path to a zapp file to a cached document instance */
    private static documentsCache;
    /** maps absolute path a fs watcher to invalidate cache once file changes */
    private static documentsWatcherMap;
    /** maps absolute path to a workspace to the absolute path of a zapp file */
    private static workspacesMap;
    /**
     * Retrieves a zapp file from the cache. Although  the file to retrieve is provided to
     * identify the file, it will not load it into the cache if not found to keep access to the
     * cache synchronous. Undefined will return, which could then be used to call loadZappDocument()
     * to load it an place it in the cache. Asynchronous file watchers can also be used to keep the cache up to date.
     * @param filePath absolute path to the zapp file
     * @returns {(Promise<ZappDocumentInstance | undefined>)} undefined if file was not cached, yet
     */
    static getZappDocument(filePath: string): ZappDocumentInstance | undefined;
    /**
     * Asynchronously loads a zapp file from the path provied.
     * @param {string} filePath absolute path to the zapp file
     * @returns {(Promise<ZappDocumentInstance | undefined>)} undefined if invalid path was provided.
     *          In case of loading errors ZappDocumentInstance will contain error messages.
     */
    static loadZappDocument(filePath: string): Promise<ZappDocumentInstance | undefined>;
    /**
     * Removes a document from the cache.
     * @param filePath absolute path to the zapp file
     */
    static clearZappDocument(filePath: string): void;
    /**
     * Clears the entire document cache including stops all file watchers.
     * Call when restarting the client and reload all files to cache.
     */
    static clearCache(): void;
    /**
     * Gets the cached ZAPP document for the workspace path provided.
     * Assumes that the document cache has been updated with the latest file, e.g. by a watcher.
     * @param workspacePath absolute path to the workspace that contains a zapp file
     * @returns {ZappDocumentInstance | undefined} undefined if invalid path was provided.
     */
    static getZappDocumentInWorkspace(workspacePath: string): ZappDocumentInstance | undefined;
    /**
     * Looks for a ZAPP file with various file extensions in the workspace directory and
     * loads the best match into the cache.
     * @param {string} workspacePath path to the workspace in which a zapp file resides
     * @returns {(Promise<ZappDocumentInstance | undefined>)} undefined if invalid path was provided.
     *          In case of loading errors ZappDocumentInstance will contain error messages.
     */
    static loadZappDocumentInWorkspace(workspacePath: string): Promise<ZappDocumentInstance | undefined>;
    /**
     * Allows retrieving a file watcher for the zapp file in a workspace to add event
     * handlers to it. Will be used for updating editors when the zapp file changes.
     * @param {string} workspacePath the absolute path to a workspace directory that contains a zapp file
     * @returns {(fs.FSWatcher | undefined)} will return undefined if no zapp file could be found
     */
    static getZappWatcherForWorkspace(workspacePath: string): fs.FSWatcher | undefined;
    /**
     * Creates a directory listener watching for zapp files to be created and
     * executes the listener once it becomes available. Will not create a watcher
     * if zapp file already exists and return false.
     *
     * @param {string} workspacePath Path in which a zapp file can be created
     * @param {(filename: string) => string} listener to be executed if file appears
     * @returns true if a watcher was created and false if not
     */
    static startZappCreationWatcherForWorkspace(workspacePath: string, listener: (filename: string) => void): boolean;
    /**
     * If a zapp file changes remove it from the cache to force a reload at the next request.
     * Watcher gets deleted once a file was changed.
     * @param {string} filePath absolute path to the file to watch.
     */
    private static addWatch;
    private static numberDocsAdded;
    private static numberDocsRemoved;
    private static numberDocsFound;
    private static numberDocsWatched;
    private static numberWatchEvents;
    /**
     * Report on cache size and stats. Mainly used for testing.
     */
    static getCacheStats(): Record<string, unknown>;
    /**
     * Normalization of absolute path names required as VS Code API returns path names
     * with lower case drive letters on Windows, but end users can specify ZAPP
     * path names with driver letter casing in the way they want. We therefore
     * index the cache on Windows always with all uppercase pathnames.
     * @param {string} absolutePath
     * @returns {string} normalized code and if Window
     * @memberof ZappDocumentsCache
     */
    private static normalizePath;
}
